/*
 * $Id$
 */

package org.apache.maven.continuum.model.project.v1_0_9.io.stax;

  //---------------------------------/
 //- Imported classes and packages -/
//---------------------------------/

import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.text.DateFormat;
import java.util.Iterator;
import java.util.Locale;
import java.util.jar.Manifest;
import javanet.staxutils.IndentingXMLStreamWriter;
import javax.xml.stream.*;
import org.apache.maven.continuum.model.project.v1_0_9.BuildDefinition;
import org.apache.maven.continuum.model.project.v1_0_9.BuildResult;
import org.apache.maven.continuum.model.project.v1_0_9.ContinuumDatabase;
import org.apache.maven.continuum.model.project.v1_0_9.Project;
import org.apache.maven.continuum.model.project.v1_0_9.ProjectDependency;
import org.apache.maven.continuum.model.project.v1_0_9.ProjectDeveloper;
import org.apache.maven.continuum.model.project.v1_0_9.ProjectGroup;
import org.apache.maven.continuum.model.project.v1_0_9.ProjectNotifier;
import org.apache.maven.continuum.model.project.v1_0_9.Schedule;
import org.apache.maven.continuum.model.scm.v1_0_9.ChangeFile;
import org.apache.maven.continuum.model.scm.v1_0_9.ChangeSet;
import org.apache.maven.continuum.model.scm.v1_0_9.ScmResult;
import org.apache.maven.continuum.model.scm.v1_0_9.SuiteResult;
import org.apache.maven.continuum.model.scm.v1_0_9.TestCaseFailure;
import org.apache.maven.continuum.model.scm.v1_0_9.TestResult;
import org.apache.maven.continuum.model.system.v1_0_9.NotificationAddress;
import org.apache.maven.continuum.model.system.v1_0_9.SystemConfiguration;
import org.codehaus.plexus.util.xml.Xpp3Dom;

/**
 * Class ContinuumStaxWriter.
 * 
 * @version $Revision$ $Date$
 */
public class ContinuumStaxWriter {


      //--------------------------/
     //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field curId
     */
    private int curId;

    /**
     * Field idMap
     */
    private java.util.Map idMap;


      //----------------/
     //- Constructors -/
    //----------------/

    public ContinuumStaxWriter() {
        idMap = new java.util.HashMap();
    } //-- org.apache.maven.continuum.model.project.v1_0_9.io.stax.ContinuumStaxWriter()


      //-----------/
     //- Methods -/
    //-----------/

    /**
     * Method write
     * 
     * @param writer
     * @param continuumDatabase
     */
    public void write(Writer writer, ContinuumDatabase continuumDatabase)
        throws java.io.IOException, XMLStreamException
    {
        XMLOutputFactory factory = XMLOutputFactory.newInstance();
        boolean supportWindowsLineEndings = false;
        if ( factory.isPropertySupported( "com.ctc.wstx.outputEscapeCr" ) )
        {
            factory.setProperty( "com.ctc.wstx.outputEscapeCr", Boolean.FALSE );
            supportWindowsLineEndings = true;
        }
        if ( factory.isPropertySupported( "org.codehaus.stax2.automaticEmptyElements" ) )
        {
            factory.setProperty( "org.codehaus.stax2.automaticEmptyElements", Boolean.FALSE );
        }
        IndentingXMLStreamWriter serializer = new IndentingXMLStreamWriter( factory.createXMLStreamWriter( writer ) );
        if ( supportWindowsLineEndings )
        {
            serializer.setNewLine( serializer.getLineSeparator() );
        }
        serializer.writeStartDocument( continuumDatabase.getModelEncoding(), "1.0" );
        writeContinuumDatabase( continuumDatabase, "continuumDatabase", serializer );
        serializer.writeEndDocument();
    } //-- void write(Writer, ContinuumDatabase) 

    /**
     * Method writeBuildDefinition
     * 
     * @param buildDefinition
     * @param serializer
     * @param tagName
     */
    private void writeBuildDefinition(BuildDefinition buildDefinition, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( buildDefinition != null )
        {
            serializer.writeStartElement( tagName );
            if ( buildDefinition.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( buildDefinition.getId() ) );
                serializer.writeEndElement();
            }
            if ( buildDefinition.isDefaultForProject() != false )
            {
                serializer.writeStartElement( "defaultForProject" );
                serializer.writeCharacters( String.valueOf( buildDefinition.isDefaultForProject() ) );
                serializer.writeEndElement();
            }
            if ( buildDefinition.getGoals() != null )
            {
                serializer.writeStartElement( "goals" );
                serializer.writeCharacters( buildDefinition.getGoals() );
                serializer.writeEndElement();
            }
            if ( buildDefinition.getArguments() != null )
            {
                serializer.writeStartElement( "arguments" );
                serializer.writeCharacters( buildDefinition.getArguments() );
                serializer.writeEndElement();
            }
            if ( buildDefinition.getBuildFile() != null )
            {
                serializer.writeStartElement( "buildFile" );
                serializer.writeCharacters( buildDefinition.getBuildFile() );
                serializer.writeEndElement();
            }
            if ( buildDefinition.getSchedule() != null )
            {
                serializer.writeStartElement( "schedule" );
                serializer.writeAttribute( "id", String.valueOf( buildDefinition.getSchedule().getId() ) );
                serializer.writeEndElement();
            }
            if ( buildDefinition.getLatestBuildId() != 0 )
            {
                serializer.writeStartElement( "latestBuildId" );
                serializer.writeCharacters( String.valueOf( buildDefinition.getLatestBuildId() ) );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeBuildDefinition(BuildDefinition, String, XMLStreamWriter) 

    /**
     * Method writeBuildResult
     * 
     * @param buildResult
     * @param serializer
     * @param tagName
     */
    private void writeBuildResult(BuildResult buildResult, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( buildResult != null )
        {
            serializer.writeStartElement( tagName );
            if ( buildResult.getProject() != null )
            {
                serializer.writeStartElement( "project" );
                serializer.writeAttribute( "id", String.valueOf( buildResult.getProject().getId() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( buildResult.getId() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getBuildNumber() != 0 )
            {
                serializer.writeStartElement( "buildNumber" );
                serializer.writeCharacters( String.valueOf( buildResult.getBuildNumber() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getState() != 0 )
            {
                serializer.writeStartElement( "state" );
                serializer.writeCharacters( String.valueOf( buildResult.getState() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getTrigger() != 0 )
            {
                serializer.writeStartElement( "trigger" );
                serializer.writeCharacters( String.valueOf( buildResult.getTrigger() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getStartTime() != 0 )
            {
                serializer.writeStartElement( "startTime" );
                serializer.writeCharacters( String.valueOf( buildResult.getStartTime() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getEndTime() != 0 )
            {
                serializer.writeStartElement( "endTime" );
                serializer.writeCharacters( String.valueOf( buildResult.getEndTime() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getError() != null )
            {
                serializer.writeStartElement( "error" );
                serializer.writeCharacters( buildResult.getError() );
                serializer.writeEndElement();
            }
            if ( buildResult.isSuccess() != false )
            {
                serializer.writeStartElement( "success" );
                serializer.writeCharacters( String.valueOf( buildResult.isSuccess() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getExitCode() != 0 )
            {
                serializer.writeStartElement( "exitCode" );
                serializer.writeCharacters( String.valueOf( buildResult.getExitCode() ) );
                serializer.writeEndElement();
            }
            if ( buildResult.getScmResult() != null )
            {
                writeScmResult( (ScmResult) buildResult.getScmResult(), "scmResult", serializer );
            }
            if ( buildResult.getTestResult() != null )
            {
                writeTestResult( (TestResult) buildResult.getTestResult(), "testResult", serializer );
            }
            serializer.writeEndElement();
        }
    } //-- void writeBuildResult(BuildResult, String, XMLStreamWriter) 

    /**
     * Method writeChangeFile
     * 
     * @param changeFile
     * @param serializer
     * @param tagName
     */
    private void writeChangeFile(ChangeFile changeFile, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( changeFile != null )
        {
            serializer.writeStartElement( tagName );
            if ( changeFile.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( changeFile.getName() );
                serializer.writeEndElement();
            }
            if ( changeFile.getRevision() != null )
            {
                serializer.writeStartElement( "revision" );
                serializer.writeCharacters( changeFile.getRevision() );
                serializer.writeEndElement();
            }
            if ( changeFile.getStatus() != null )
            {
                serializer.writeStartElement( "status" );
                serializer.writeCharacters( changeFile.getStatus() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeChangeFile(ChangeFile, String, XMLStreamWriter) 

    /**
     * Method writeChangeSet
     * 
     * @param changeSet
     * @param serializer
     * @param tagName
     */
    private void writeChangeSet(ChangeSet changeSet, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( changeSet != null )
        {
            serializer.writeStartElement( tagName );
            if ( changeSet.getAuthor() != null )
            {
                serializer.writeStartElement( "author" );
                serializer.writeCharacters( changeSet.getAuthor() );
                serializer.writeEndElement();
            }
            if ( changeSet.getComment() != null )
            {
                serializer.writeStartElement( "comment" );
                serializer.writeCharacters( changeSet.getComment() );
                serializer.writeEndElement();
            }
            if ( changeSet.getDate() != 0 )
            {
                serializer.writeStartElement( "date" );
                serializer.writeCharacters( String.valueOf( changeSet.getDate() ) );
                serializer.writeEndElement();
            }
            if ( changeSet.getFiles() != null && changeSet.getFiles().size() > 0 )
            {
                serializer.writeStartElement( "files" );
                for ( Iterator iter = changeSet.getFiles().iterator(); iter.hasNext(); )
                {
                    ChangeFile o = (ChangeFile) iter.next();
                    writeChangeFile( o, "file", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeChangeSet(ChangeSet, String, XMLStreamWriter) 

    /**
     * Method writeContinuumDatabase
     * 
     * @param continuumDatabase
     * @param serializer
     * @param tagName
     */
    private void writeContinuumDatabase(ContinuumDatabase continuumDatabase, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( continuumDatabase != null )
        {
            serializer.writeStartElement( tagName );
            if ( continuumDatabase.getProjectGroups() != null && continuumDatabase.getProjectGroups().size() > 0 )
            {
                serializer.writeStartElement( "projectGroups" );
                for ( Iterator iter = continuumDatabase.getProjectGroups().iterator(); iter.hasNext(); )
                {
                    ProjectGroup o = (ProjectGroup) iter.next();
                    writeProjectGroup( o, "projectGroup", serializer );
                }
                serializer.writeEndElement();
            }
            if ( continuumDatabase.getSystemConfiguration() != null )
            {
                writeSystemConfiguration( (SystemConfiguration) continuumDatabase.getSystemConfiguration(), "systemConfiguration", serializer );
            }
            if ( continuumDatabase.getSchedules() != null && continuumDatabase.getSchedules().size() > 0 )
            {
                serializer.writeStartElement( "schedules" );
                for ( Iterator iter = continuumDatabase.getSchedules().iterator(); iter.hasNext(); )
                {
                    Schedule o = (Schedule) iter.next();
                    writeSchedule( o, "schedule", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeContinuumDatabase(ContinuumDatabase, String, XMLStreamWriter) 

    /**
     * Method writeDom
     * 
     * @param dom
     * @param serializer
     */
    public void writeDom(Xpp3Dom dom, XMLStreamWriter serializer)
        throws XMLStreamException
    {
        serializer.writeStartElement( dom.getName() );
        String[] attributeNames = dom.getAttributeNames();
        for ( int i = 0; i < attributeNames.length; i++ )
        {
            String attributeName = attributeNames[i];
            serializer.writeAttribute( attributeName, dom.getAttribute( attributeName ) );
        }
        Xpp3Dom[] children = dom.getChildren();
        for ( int i = 0; i < children.length; i++ )
        {
            writeDom( children[i], serializer );
        }
        String value = dom.getValue();
        if ( value != null )
        {
            serializer.writeCharacters( value );
        }
        serializer.writeEndElement();
    } //-- void writeDom(Xpp3Dom, XMLStreamWriter) 

    /**
     * Method writeNotificationAddress
     * 
     * @param notificationAddress
     * @param serializer
     * @param tagName
     */
    private void writeNotificationAddress(NotificationAddress notificationAddress, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( notificationAddress != null )
        {
            serializer.writeStartElement( tagName );
            if ( notificationAddress.getType() != null && !notificationAddress.getType().equals( "mail" ) )
            {
                serializer.writeStartElement( "type" );
                serializer.writeCharacters( notificationAddress.getType() );
                serializer.writeEndElement();
            }
            if ( notificationAddress.getAddress() != null )
            {
                serializer.writeStartElement( "address" );
                serializer.writeCharacters( notificationAddress.getAddress() );
                serializer.writeEndElement();
            }
            if ( notificationAddress.getConfiguration() != null && notificationAddress.getConfiguration().size() > 0 )
            {
                serializer.writeStartElement( "configuration" );
                for ( Iterator iter = notificationAddress.getConfiguration().keySet().iterator(); iter.hasNext(); )
                {
                    String key = (String) iter.next();
                    String value = (String) notificationAddress.getConfiguration().get( key );
                    serializer.writeStartElement( "" + key + "" );
                    serializer.writeCharacters( value );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeNotificationAddress(NotificationAddress, String, XMLStreamWriter) 

    /**
     * Method writeProject
     * 
     * @param project
     * @param serializer
     * @param tagName
     */
    private void writeProject(Project project, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( project != null )
        {
            serializer.writeStartElement( tagName );
            if ( project.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( project.getId() ) );
                serializer.writeEndElement();
            }
            if ( project.getGroupId() != null )
            {
                serializer.writeStartElement( "groupId" );
                serializer.writeCharacters( project.getGroupId() );
                serializer.writeEndElement();
            }
            if ( project.getArtifactId() != null )
            {
                serializer.writeStartElement( "artifactId" );
                serializer.writeCharacters( project.getArtifactId() );
                serializer.writeEndElement();
            }
            if ( project.getExecutorId() != null )
            {
                serializer.writeStartElement( "executorId" );
                serializer.writeCharacters( project.getExecutorId() );
                serializer.writeEndElement();
            }
            if ( project.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( project.getName() );
                serializer.writeEndElement();
            }
            if ( project.getDescription() != null )
            {
                serializer.writeStartElement( "description" );
                serializer.writeCharacters( project.getDescription() );
                serializer.writeEndElement();
            }
            if ( project.getUrl() != null )
            {
                serializer.writeStartElement( "url" );
                serializer.writeCharacters( project.getUrl() );
                serializer.writeEndElement();
            }
            if ( project.getScmUrl() != null )
            {
                serializer.writeStartElement( "scmUrl" );
                serializer.writeCharacters( project.getScmUrl() );
                serializer.writeEndElement();
            }
            if ( project.getScmTag() != null )
            {
                serializer.writeStartElement( "scmTag" );
                serializer.writeCharacters( project.getScmTag() );
                serializer.writeEndElement();
            }
            if ( project.getScmUsername() != null )
            {
                serializer.writeStartElement( "scmUsername" );
                serializer.writeCharacters( project.getScmUsername() );
                serializer.writeEndElement();
            }
            if ( project.getScmPassword() != null )
            {
                serializer.writeStartElement( "scmPassword" );
                serializer.writeCharacters( project.getScmPassword() );
                serializer.writeEndElement();
            }
            if ( project.getVersion() != null )
            {
                serializer.writeStartElement( "version" );
                serializer.writeCharacters( project.getVersion() );
                serializer.writeEndElement();
            }
            if ( project.getState() != 1 )
            {
                serializer.writeStartElement( "state" );
                serializer.writeCharacters( String.valueOf( project.getState() ) );
                serializer.writeEndElement();
            }
            if ( project.getOldState() != 0 )
            {
                serializer.writeStartElement( "oldState" );
                serializer.writeCharacters( String.valueOf( project.getOldState() ) );
                serializer.writeEndElement();
            }
            if ( project.getLatestBuildId() != 0 )
            {
                serializer.writeStartElement( "latestBuildId" );
                serializer.writeCharacters( String.valueOf( project.getLatestBuildId() ) );
                serializer.writeEndElement();
            }
            if ( project.getBuildNumber() != 0 )
            {
                serializer.writeStartElement( "buildNumber" );
                serializer.writeCharacters( String.valueOf( project.getBuildNumber() ) );
                serializer.writeEndElement();
            }
            if ( project.getWorkingDirectory() != null )
            {
                serializer.writeStartElement( "workingDirectory" );
                serializer.writeCharacters( project.getWorkingDirectory() );
                serializer.writeEndElement();
            }
            if ( project.getBuildResults() != null && project.getBuildResults().size() > 0 )
            {
                serializer.writeStartElement( "buildResults" );
                for ( Iterator iter = project.getBuildResults().iterator(); iter.hasNext(); )
                {
                    BuildResult o = (BuildResult) iter.next();
                    writeBuildResult( o, "buildResult", serializer );
                }
                serializer.writeEndElement();
            }
            if ( project.getCheckoutResult() != null )
            {
                writeScmResult( (ScmResult) project.getCheckoutResult(), "checkoutResult", serializer );
            }
            if ( project.getDevelopers() != null && project.getDevelopers().size() > 0 )
            {
                serializer.writeStartElement( "developers" );
                for ( Iterator iter = project.getDevelopers().iterator(); iter.hasNext(); )
                {
                    ProjectDeveloper o = (ProjectDeveloper) iter.next();
                    writeProjectDeveloper( o, "developer", serializer );
                }
                serializer.writeEndElement();
            }
            if ( project.getParent() != null )
            {
                writeProjectDependency( (ProjectDependency) project.getParent(), "parent", serializer );
            }
            if ( project.getDependencies() != null && project.getDependencies().size() > 0 )
            {
                serializer.writeStartElement( "dependencies" );
                for ( Iterator iter = project.getDependencies().iterator(); iter.hasNext(); )
                {
                    ProjectDependency o = (ProjectDependency) iter.next();
                    writeProjectDependency( o, "dependency", serializer );
                }
                serializer.writeEndElement();
            }
            if ( project.getProjectGroup() != null )
            {
                serializer.writeStartElement( "projectGroup" );
                serializer.writeAttribute( "id", String.valueOf( project.getProjectGroup().getId() ) );
                serializer.writeEndElement();
            }
            if ( project.getNotifiers() != null && project.getNotifiers().size() > 0 )
            {
                serializer.writeStartElement( "notifiers" );
                for ( Iterator iter = project.getNotifiers().iterator(); iter.hasNext(); )
                {
                    ProjectNotifier o = (ProjectNotifier) iter.next();
                    writeProjectNotifier( o, "notifier", serializer );
                }
                serializer.writeEndElement();
            }
            if ( project.getBuildDefinitions() != null && project.getBuildDefinitions().size() > 0 )
            {
                serializer.writeStartElement( "buildDefinitions" );
                for ( Iterator iter = project.getBuildDefinitions().iterator(); iter.hasNext(); )
                {
                    BuildDefinition o = (BuildDefinition) iter.next();
                    writeBuildDefinition( o, "buildDefinition", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeProject(Project, String, XMLStreamWriter) 

    /**
     * Method writeProjectDependency
     * 
     * @param projectDependency
     * @param serializer
     * @param tagName
     */
    private void writeProjectDependency(ProjectDependency projectDependency, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( projectDependency != null )
        {
            serializer.writeStartElement( tagName );
            if ( projectDependency.getGroupId() != null )
            {
                serializer.writeStartElement( "groupId" );
                serializer.writeCharacters( projectDependency.getGroupId() );
                serializer.writeEndElement();
            }
            if ( projectDependency.getArtifactId() != null )
            {
                serializer.writeStartElement( "artifactId" );
                serializer.writeCharacters( projectDependency.getArtifactId() );
                serializer.writeEndElement();
            }
            if ( projectDependency.getVersion() != null )
            {
                serializer.writeStartElement( "version" );
                serializer.writeCharacters( projectDependency.getVersion() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeProjectDependency(ProjectDependency, String, XMLStreamWriter) 

    /**
     * Method writeProjectDeveloper
     * 
     * @param projectDeveloper
     * @param serializer
     * @param tagName
     */
    private void writeProjectDeveloper(ProjectDeveloper projectDeveloper, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( projectDeveloper != null )
        {
            serializer.writeStartElement( tagName );
            if ( projectDeveloper.getScmId() != null )
            {
                serializer.writeStartElement( "scmId" );
                serializer.writeCharacters( projectDeveloper.getScmId() );
                serializer.writeEndElement();
            }
            if ( projectDeveloper.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( projectDeveloper.getName() );
                serializer.writeEndElement();
            }
            if ( projectDeveloper.getEmail() != null )
            {
                serializer.writeStartElement( "email" );
                serializer.writeCharacters( projectDeveloper.getEmail() );
                serializer.writeEndElement();
            }
            if ( projectDeveloper.getContinuumId() != 0 )
            {
                serializer.writeStartElement( "continuumId" );
                serializer.writeCharacters( String.valueOf( projectDeveloper.getContinuumId() ) );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeProjectDeveloper(ProjectDeveloper, String, XMLStreamWriter) 

    /**
     * Method writeProjectGroup
     * 
     * @param projectGroup
     * @param serializer
     * @param tagName
     */
    private void writeProjectGroup(ProjectGroup projectGroup, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( projectGroup != null )
        {
            serializer.writeStartElement( tagName );
            if ( projectGroup.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( projectGroup.getId() ) );
                serializer.writeEndElement();
            }
            if ( projectGroup.getGroupId() != null )
            {
                serializer.writeStartElement( "groupId" );
                serializer.writeCharacters( projectGroup.getGroupId() );
                serializer.writeEndElement();
            }
            if ( projectGroup.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( projectGroup.getName() );
                serializer.writeEndElement();
            }
            if ( projectGroup.getDescription() != null )
            {
                serializer.writeStartElement( "description" );
                serializer.writeCharacters( projectGroup.getDescription() );
                serializer.writeEndElement();
            }
            if ( projectGroup.getProjects() != null && projectGroup.getProjects().size() > 0 )
            {
                serializer.writeStartElement( "projects" );
                for ( Iterator iter = projectGroup.getProjects().iterator(); iter.hasNext(); )
                {
                    Project o = (Project) iter.next();
                    writeProject( o, "project", serializer );
                }
                serializer.writeEndElement();
            }
            if ( projectGroup.getNotifiers() != null && projectGroup.getNotifiers().size() > 0 )
            {
                serializer.writeStartElement( "notifiers" );
                for ( Iterator iter = projectGroup.getNotifiers().iterator(); iter.hasNext(); )
                {
                    ProjectNotifier o = (ProjectNotifier) iter.next();
                    writeProjectNotifier( o, "notifier", serializer );
                }
                serializer.writeEndElement();
            }
            if ( projectGroup.getBuildDefinitions() != null && projectGroup.getBuildDefinitions().size() > 0 )
            {
                serializer.writeStartElement( "buildDefinitions" );
                for ( Iterator iter = projectGroup.getBuildDefinitions().iterator(); iter.hasNext(); )
                {
                    BuildDefinition o = (BuildDefinition) iter.next();
                    writeBuildDefinition( o, "buildDefinition", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeProjectGroup(ProjectGroup, String, XMLStreamWriter) 

    /**
     * Method writeProjectNotifier
     * 
     * @param projectNotifier
     * @param serializer
     * @param tagName
     */
    private void writeProjectNotifier(ProjectNotifier projectNotifier, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( projectNotifier != null )
        {
            serializer.writeStartElement( tagName );
            if ( projectNotifier.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( projectNotifier.getId() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.getType() != null && !projectNotifier.getType().equals( "mail" ) )
            {
                serializer.writeStartElement( "type" );
                serializer.writeCharacters( projectNotifier.getType() );
                serializer.writeEndElement();
            }
            if ( projectNotifier.getFrom() != 0 )
            {
                serializer.writeStartElement( "from" );
                serializer.writeCharacters( String.valueOf( projectNotifier.getFrom() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.isEnabled() != true )
            {
                serializer.writeStartElement( "enabled" );
                serializer.writeCharacters( String.valueOf( projectNotifier.isEnabled() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.getRecipientType() != 0 )
            {
                serializer.writeStartElement( "recipientType" );
                serializer.writeCharacters( String.valueOf( projectNotifier.getRecipientType() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.isSendOnSuccess() != true )
            {
                serializer.writeStartElement( "sendOnSuccess" );
                serializer.writeCharacters( String.valueOf( projectNotifier.isSendOnSuccess() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.isSendOnFailure() != true )
            {
                serializer.writeStartElement( "sendOnFailure" );
                serializer.writeCharacters( String.valueOf( projectNotifier.isSendOnFailure() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.isSendOnError() != true )
            {
                serializer.writeStartElement( "sendOnError" );
                serializer.writeCharacters( String.valueOf( projectNotifier.isSendOnError() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.isSendOnWarning() != true )
            {
                serializer.writeStartElement( "sendOnWarning" );
                serializer.writeCharacters( String.valueOf( projectNotifier.isSendOnWarning() ) );
                serializer.writeEndElement();
            }
            if ( projectNotifier.getConfiguration() != null && projectNotifier.getConfiguration().size() > 0 )
            {
                serializer.writeStartElement( "configuration" );
                for ( Iterator iter = projectNotifier.getConfiguration().keySet().iterator(); iter.hasNext(); )
                {
                    String key = (String) iter.next();
                    String value = (String) projectNotifier.getConfiguration().get( key );
                    serializer.writeStartElement( "" + key + "" );
                    serializer.writeCharacters( value );
                    serializer.writeEndElement();
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeProjectNotifier(ProjectNotifier, String, XMLStreamWriter) 

    /**
     * Method writeSchedule
     * 
     * @param schedule
     * @param serializer
     * @param tagName
     */
    private void writeSchedule(Schedule schedule, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( schedule != null )
        {
            serializer.writeStartElement( tagName );
            if ( schedule.getId() != 0 )
            {
                serializer.writeStartElement( "id" );
                serializer.writeCharacters( String.valueOf( schedule.getId() ) );
                serializer.writeEndElement();
            }
            if ( schedule.isActive() != false )
            {
                serializer.writeStartElement( "active" );
                serializer.writeCharacters( String.valueOf( schedule.isActive() ) );
                serializer.writeEndElement();
            }
            if ( schedule.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( schedule.getName() );
                serializer.writeEndElement();
            }
            if ( schedule.getDescription() != null )
            {
                serializer.writeStartElement( "description" );
                serializer.writeCharacters( schedule.getDescription() );
                serializer.writeEndElement();
            }
            if ( schedule.getDelay() != 0 )
            {
                serializer.writeStartElement( "delay" );
                serializer.writeCharacters( String.valueOf( schedule.getDelay() ) );
                serializer.writeEndElement();
            }
            if ( schedule.getMaxJobExecutionTime() != 3600 )
            {
                serializer.writeStartElement( "maxJobExecutionTime" );
                serializer.writeCharacters( String.valueOf( schedule.getMaxJobExecutionTime() ) );
                serializer.writeEndElement();
            }
            if ( schedule.getCronExpression() != null )
            {
                serializer.writeStartElement( "cronExpression" );
                serializer.writeCharacters( schedule.getCronExpression() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeSchedule(Schedule, String, XMLStreamWriter) 

    /**
     * Method writeScmResult
     * 
     * @param scmResult
     * @param serializer
     * @param tagName
     */
    private void writeScmResult(ScmResult scmResult, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( scmResult != null )
        {
            serializer.writeStartElement( tagName );
            if ( scmResult.isSuccess() != false )
            {
                serializer.writeStartElement( "success" );
                serializer.writeCharacters( String.valueOf( scmResult.isSuccess() ) );
                serializer.writeEndElement();
            }
            if ( scmResult.getCommandLine() != null )
            {
                serializer.writeStartElement( "commandLine" );
                serializer.writeCharacters( scmResult.getCommandLine() );
                serializer.writeEndElement();
            }
            if ( scmResult.getProviderMessage() != null )
            {
                serializer.writeStartElement( "providerMessage" );
                serializer.writeCharacters( scmResult.getProviderMessage() );
                serializer.writeEndElement();
            }
            if ( scmResult.getCommandOutput() != null )
            {
                serializer.writeStartElement( "commandOutput" );
                serializer.writeCharacters( scmResult.getCommandOutput() );
                serializer.writeEndElement();
            }
            if ( scmResult.getException() != null )
            {
                serializer.writeStartElement( "exception" );
                serializer.writeCharacters( scmResult.getException() );
                serializer.writeEndElement();
            }
            if ( scmResult.getChanges() != null && scmResult.getChanges().size() > 0 )
            {
                serializer.writeStartElement( "changes" );
                for ( Iterator iter = scmResult.getChanges().iterator(); iter.hasNext(); )
                {
                    ChangeSet o = (ChangeSet) iter.next();
                    writeChangeSet( o, "change", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeScmResult(ScmResult, String, XMLStreamWriter) 

    /**
     * Method writeSuiteResult
     * 
     * @param suiteResult
     * @param serializer
     * @param tagName
     */
    private void writeSuiteResult(SuiteResult suiteResult, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( suiteResult != null )
        {
            serializer.writeStartElement( tagName );
            if ( suiteResult.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( suiteResult.getName() );
                serializer.writeEndElement();
            }
            if ( suiteResult.getTestCount() != 0 )
            {
                serializer.writeStartElement( "testCount" );
                serializer.writeCharacters( String.valueOf( suiteResult.getTestCount() ) );
                serializer.writeEndElement();
            }
            if ( suiteResult.getFailureCount() != 0 )
            {
                serializer.writeStartElement( "failureCount" );
                serializer.writeCharacters( String.valueOf( suiteResult.getFailureCount() ) );
                serializer.writeEndElement();
            }
            if ( suiteResult.getTotalTime() != 0 )
            {
                serializer.writeStartElement( "totalTime" );
                serializer.writeCharacters( String.valueOf( suiteResult.getTotalTime() ) );
                serializer.writeEndElement();
            }
            if ( suiteResult.getFailures() != null && suiteResult.getFailures().size() > 0 )
            {
                serializer.writeStartElement( "failures" );
                for ( Iterator iter = suiteResult.getFailures().iterator(); iter.hasNext(); )
                {
                    TestCaseFailure o = (TestCaseFailure) iter.next();
                    writeTestCaseFailure( o, "failure", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeSuiteResult(SuiteResult, String, XMLStreamWriter) 

    /**
     * Method writeSystemConfiguration
     * 
     * @param systemConfiguration
     * @param serializer
     * @param tagName
     */
    private void writeSystemConfiguration(SystemConfiguration systemConfiguration, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( systemConfiguration != null )
        {
            serializer.writeStartElement( tagName );
            if ( systemConfiguration.isGuestAccountEnabled() != true )
            {
                serializer.writeStartElement( "guestAccountEnabled" );
                serializer.writeCharacters( String.valueOf( systemConfiguration.isGuestAccountEnabled() ) );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getDefaultScheduleDescription() != null && !systemConfiguration.getDefaultScheduleDescription().equals( "Run hourly" ) )
            {
                serializer.writeStartElement( "defaultScheduleDescription" );
                serializer.writeCharacters( systemConfiguration.getDefaultScheduleDescription() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getDefaultScheduleCronExpression() != null && !systemConfiguration.getDefaultScheduleCronExpression().equals( "0 0 * * * ?" ) )
            {
                serializer.writeStartElement( "defaultScheduleCronExpression" );
                serializer.writeCharacters( systemConfiguration.getDefaultScheduleCronExpression() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getWorkingDirectory() != null && !systemConfiguration.getWorkingDirectory().equals( "working-directory" ) )
            {
                serializer.writeStartElement( "workingDirectory" );
                serializer.writeCharacters( systemConfiguration.getWorkingDirectory() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getBuildOutputDirectory() != null && !systemConfiguration.getBuildOutputDirectory().equals( "build-output-directory" ) )
            {
                serializer.writeStartElement( "buildOutputDirectory" );
                serializer.writeCharacters( systemConfiguration.getBuildOutputDirectory() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getDeploymentRepositoryDirectory() != null )
            {
                serializer.writeStartElement( "deploymentRepositoryDirectory" );
                serializer.writeCharacters( systemConfiguration.getDeploymentRepositoryDirectory() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.getBaseUrl() != null )
            {
                serializer.writeStartElement( "baseUrl" );
                serializer.writeCharacters( systemConfiguration.getBaseUrl() );
                serializer.writeEndElement();
            }
            if ( systemConfiguration.isInitialized() != false )
            {
                serializer.writeStartElement( "initialized" );
                serializer.writeCharacters( String.valueOf( systemConfiguration.isInitialized() ) );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeSystemConfiguration(SystemConfiguration, String, XMLStreamWriter) 

    /**
     * Method writeTestCaseFailure
     * 
     * @param testCaseFailure
     * @param serializer
     * @param tagName
     */
    private void writeTestCaseFailure(TestCaseFailure testCaseFailure, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( testCaseFailure != null )
        {
            serializer.writeStartElement( tagName );
            if ( testCaseFailure.getName() != null )
            {
                serializer.writeStartElement( "name" );
                serializer.writeCharacters( testCaseFailure.getName() );
                serializer.writeEndElement();
            }
            if ( testCaseFailure.getException() != null )
            {
                serializer.writeStartElement( "exception" );
                serializer.writeCharacters( testCaseFailure.getException() );
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeTestCaseFailure(TestCaseFailure, String, XMLStreamWriter) 

    /**
     * Method writeTestResult
     * 
     * @param testResult
     * @param serializer
     * @param tagName
     */
    private void writeTestResult(TestResult testResult, String tagName, XMLStreamWriter serializer)
        throws java.io.IOException, XMLStreamException
    {
        if ( testResult != null )
        {
            serializer.writeStartElement( tagName );
            if ( testResult.getTestCount() != 0 )
            {
                serializer.writeStartElement( "testCount" );
                serializer.writeCharacters( String.valueOf( testResult.getTestCount() ) );
                serializer.writeEndElement();
            }
            if ( testResult.getFailureCount() != 0 )
            {
                serializer.writeStartElement( "failureCount" );
                serializer.writeCharacters( String.valueOf( testResult.getFailureCount() ) );
                serializer.writeEndElement();
            }
            if ( testResult.getTotalTime() != 0 )
            {
                serializer.writeStartElement( "totalTime" );
                serializer.writeCharacters( String.valueOf( testResult.getTotalTime() ) );
                serializer.writeEndElement();
            }
            if ( testResult.getSuiteResults() != null && testResult.getSuiteResults().size() > 0 )
            {
                serializer.writeStartElement( "suiteResults" );
                for ( Iterator iter = testResult.getSuiteResults().iterator(); iter.hasNext(); )
                {
                    SuiteResult o = (SuiteResult) iter.next();
                    writeSuiteResult( o, "suiteResult", serializer );
                }
                serializer.writeEndElement();
            }
            serializer.writeEndElement();
        }
    } //-- void writeTestResult(TestResult, String, XMLStreamWriter) 


}
